package crawler;

import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import outputFormat.AsosProduct;
import outputFormat.LVProduct;
import utility.ExcelManager;
import utility.SiteManager;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import utility.ErrorLog;

public class EluxuryCrawler {

	// file to write the output on
	private static String outputFile = "excelfile/lv.xls";
	// site url
	private static String siteurl = "http://www.eluxury.com";
	// base url
	private static String baseurl = "http://www.eluxury.com/estore/brands/louis_vuitton/w05/";
	// page to start crawl content
	private static String startpage1 = "http://www.eluxury.com/estore/brands/louis_vuitton/w05/collection.jsp?catId=cat20427&brand=3250";
	private static String startpage2 = "http://www.eluxury.com/estore/brands/louis_vuitton/w05/collection.jsp?catId=cat20429&brand=3250";

	private static String start_women = "http://www.eluxury.com/estore/brands/louis_vuitton/w05/collection.jsp?catId=cat20427&brand=3250";
	private static String start_men = "http://www.eluxury.com/estore/brands/louis_vuitton/w05/collection.jsp?catId=cat20429&brand=3250";

	// gender of the product
	private static String gender;
	// image file save to fold
	private static String imgFold = "LV/";

	private static String brand = "Louis Vuitton";

	private static String priceUnit = "USD";

	private static int prodCount = 1;
	private static int errorCount = 0;

	private static int startpos = 0; // 276//714 for women

	private static ErrorLog errorLog;

	// main to start crawler
	public static void main(String[] args) {
		LVProduct lv = new LVProduct();
		ExcelManager.prepareFile(outputFile, lv);

		//gender = "women";
		//crawlContent(start_women);

		 gender = "men";
		 crawlContent(start_men);

		// crawlProduct("test", "test","test",
		// "http://www.eluxury.com/estore/browse/product_detail.jsp?id=11871207"
		// );

		System.out.println("All finished!");
		System.out.println("Number of errors: " + errorCount);

	}

	private static void crawlContent(String url) {
		// Category list to crawl content on each
		Hashtable<String, String> categories = new Hashtable<String, String>();

		// save the page to a String
		String doc = SiteManager.getDocumentFromURL(url,
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

		// find current category name
		Pattern cateP = Pattern
				.compile("<span class=\"leftNavSubCategoryListOn\">([^<]*?)</span>");
		Matcher cateM = cateP.matcher(doc);
		while (cateM.find()) {
			String cateName = cateM.group(1);
			System.out.println("Found category: " + cateName + ", URL: " + url);
			categories.put(cateName, url);
		}

		// find other category
		cateP = Pattern
				.compile("<a href=\"([^\"]*?)\" class=\"leftNavSubCategoryListOff\">([^<]*?)</a>");
		cateM = cateP.matcher(doc);
		while (cateM.find()) {
			String cateURL = cateM.group(1);
			cateURL = urlCorrection(cateURL, baseurl); // correct the url by
			// adding the prefix,
			// and remove the amp;
			String cateName = cateM.group(2);
			System.out.println("Found category: " + cateName + ", URL: "
					+ cateURL);
			categories.put(cateName, cateURL);
		}

		System.out.println("Searching categories process finished.\n");

		// System.out.println(categories.values());

		// for each category crawl content find different styles
		for (String cateName : categories.keySet()) {
			// System.out.println("Begin searching style...");
			findStyle(cateName, categories.get(cateName));
		}

	}

	private static void findStyle(String cateName, String cateURL) {
		// System.out.println(cateName+ " : " + cateURL);

		// save the page to a String
		String doc = SiteManager.getDocumentFromURL(cateURL,
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

		// find styles
		Pattern styleP = Pattern
				.compile("<a href=\"([^\"]*?)\" class=\"mainSubSubCategoryListOff\" onmouseout=\"([^\"]*?)\" onmouseover=\"([^\"]*?)\">([^>]*?)</a>");
		Matcher styleM = styleP.matcher(doc);
		while (styleM.find()) {
			String styleName = styleM.group(4);
			String styleURL = styleM.group(1);
			styleURL = urlCorrection(styleURL, baseurl);
			System.out
					.println("++++++++++++++ begin searching a style ++++++++++++++++++++++");
			System.out.println("Found style name: " + styleName
					+ " Style url: " + styleURL);
			findProduct(cateName, styleName, styleURL);
			System.out.println("--------------- end of style: " + styleName
					+ " ----------------------------\n\n\n");
			try {
				System.out.println("Begin sleep");
				Thread.sleep(8000);
				System.out.println("End sleep");
			} catch (Exception e) {
				System.out.println(e.toString());
			}
		}
	}

	private static void findProduct(String cateName, String styleName,
			String styleURL) {
		// save the page to a String
		String doc = SiteManager.getDocumentFromURL(styleURL,
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

		// find products
		Pattern productP = Pattern
				.compile("<a href=\"([^\"]*?)\" class=\"mainProductListOff\" onmouseover=\"([^\"]*?)\">([^>]*?)</a>");
		Matcher productM = productP.matcher(doc);
		while (productM.find()) {
			String productName = productM.group(3);
			String productURL = productM.group(1);
			productURL = urlCorrection(productURL, siteurl);
			System.out
					.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
			System.out.println("Found product name: " + productName
					+ "  product url:s " + productURL);
			crawlProduct(cateName, styleName, productName, productURL);
			System.out
					.println("---------------------------------------------------------------\n\n");
		}

		// find next page
		Pattern nextP = Pattern
				.compile("<a href=\"([^\"]*?)\" class=\"mainPageLinkOff\"><img width=\"10\" height=\"23\" src=\"/estore/assets_server/global/buttons/btn_next.gif\" border=\"0\"></img></a>");
		Matcher nextM = nextP.matcher(doc);
		// System.out.println("Finding next page!");
		// System.out.println(doc);
		while (nextM.find()) {
			String pageurl = nextM.group(1);
			pageurl = urlCorrection(pageurl, siteurl);
			// System.out.println("Next page: "+pageurl);
			findProduct(cateName, styleName, pageurl);
		}
		// System.out.println("Find next page end!");
	}

	private static void crawlProduct(String cateName, String styleName,
			String productName, String productURL) {
		System.out.println("productCount: " + prodCount
				+ "---------------------");
		// if (startpos <= prodCount) {
		if (true) {
			// System.out.println(LVProduct.idCount + " : " + productURL);
			System.out.println("Url : " + productURL);
			System.out.println("Crawling one product ...");
			// save the page to a String
			String doc = SiteManager.getDocumentFromURL(productURL,
					"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");
			// System.out.print(doc);
			// Pattern nameP =
			// Pattern.compile("<td class=\"desc\"><span>([^<]*?)</span></td>");
			// Matcher nameM = nameP.matcher(doc);
			// Pattern priceP =
			// Pattern.compile("<td class=\"price\">\\$([^<]*?)</td>");
			// Matcher priceM = priceP.matcher(doc);
			// Pattern imageP = Pattern.compile(
			// "<img(.*?)class=\"bigprod\" src=\"([^\"]*?)\"([^>]*?)>");
			Pattern imageP = Pattern
					.compile("class=\"bigprod\" src=\"([^\"]*?)\""); // changed
			Matcher imageM = imageP.matcher(doc);
			// regex changed by maoanz
			// Pattern descP = Pattern
			// .compile(
			// "<td class=\"tabresult\"><span>(<ul>((\\s*?)<li>([^>]*?)</li>(\\s*?))*?</(ul|UL)>)?([^>])*?</span><span class=\"pdDesc\"></span></td>"
			// );
			Pattern descP = Pattern
					.compile("<td class=\"tabresult\"><span>([^^]*?)</span>");

			Matcher descM = descP.matcher(doc);
			// if (imageM.find() && descM.find()) {

			// changed by maoanz

			String name = productName.substring(0, productName.indexOf(","));
			String price = productName.substring(productName.indexOf("$") + 1);
			String image = "";
			// String desc = descM.group(1) + descM.group(2);

			String desc = ""; // changed by maoanz
			if (descM.find()) {
				desc = descM.group(1);
			} else {
				desc = "error";
				System.out.println("An error occured");
				errorCount++;
			}

			if (imageM.find()) {
				image = imageM.group(1);
			} else {
				image = "error";
				System.out.println("An error occured");
				errorCount++;
			}

			try {
				image = SiteManager
						.saveImage(image, imgFold, LVProduct.idCount);
			} catch (Exception e) {
				e.printStackTrace();
			}
			System.out.println("Product name: " + name + "| price: " + price
					+ "| image: " + image + "| desc: " + desc);
			LVProduct lv = new LVProduct(name, image, brand, price, priceUnit,
					gender, desc, cateName, styleName, productURL);
			ExcelManager.writeToFile(lv);
			System.out.println("One product inserted :-)");
			prodCount++;

			System.out.println("\n");
		}
		// prodCount++;
	}

	private static String urlCorrection(String url, String preurl) {
		url = preurl + url;
		url = url.substring(0, url.indexOf(";"))
				+ url.substring(url.indexOf("?"));
		url = url.replaceAll("amp;", "");
		return url;
	}
}
